@@ -24,7 +24,7 @@ from twisted.python.failure import Failure
 from twisted.web.util import formatFailure
 from twisted.python.reflect import accumulateClassList
 
-from buildbot import interfaces, locks, util
+from buildbot import interfaces, locks, util, config
 from buildbot.status import progress
 from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, SKIPPED, \
      EXCEPTION, RETRY, worst_status
@@ -39,12 +39,25 @@ class RemoteCommand(pb.Referenceable):
     _commandCounter = 0
 
     active = False
+    rc = None
+    debug = False
 
-    def __init__(self, remote_command, args, ignore_updates=False):
+    def __init__(self, remote_command, args, ignore_updates=False, collectStdout=False):
+        self.logs = {}
+        self.delayedLogs = {}
+        self._closeWhenFinished = {}
+        self.collectStdout = collectStdout
+        self.stdout = ''
+
+        self._startTime = None
+        self._remoteElapsed = None
         self.remote_command = remote_command
         self.args = args
         self.ignore_updates = ignore_updates
 
+    def __repr__(self):
+        return "<RemoteCommand '%s' at %d>" % (self.remote_command, id(self))
+
     def run(self, step, remote):
         self.active = True
         self.step = step
@@ -72,16 +85,30 @@ class RemoteCommand(pb.Referenceable):
         # when our parent Step calls our .lostRemote() method.
         return self.deferred
 
+    def useLog(self, loog, closeWhenFinished=False, logfileName=None):
+        assert interfaces.ILogFile.providedBy(loog)
+        if not logfileName:
+            logfileName = loog.getName()
+        assert logfileName not in self.logs
+        assert logfileName not in self.delayedLogs
+        self.logs[logfileName] = loog
+        self._closeWhenFinished[logfileName] = closeWhenFinished
+
+    def useLogDelayed(self, logfileName, activateCallBack, closeWhenFinished=False):
+        assert logfileName not in self.logs
+        assert logfileName not in self.delayedLogs
+        self.delayedLogs[logfileName] = (activateCallBack, closeWhenFinished)
+
     def _start(self):
-        # Allow use of WithProperties in logfile path names.
-        cmd_args = self.args
+        self.updates = {}
+        self._startTime = util.now()
 
         # This method only initiates the remote command.
         # We will receive remote_update messages as the command runs.
         # We will get a single remote_complete when it finishes.
         # We should fire self.deferred when the command is done.
         d = self.remote.callRemote("startCommand", self, self.commandID,
-                                   self.remote_command, cmd_args)
+                                   self.remote_command, self.args)
         return d
 
     def _finished(self, failure=None):
@@ -152,9 +179,6 @@ class RemoteCommand(pb.Referenceable):
                 max_updatenum = num
         return max_updatenum
 
-    def remoteUpdate(self, update):
-        raise NotImplementedError("You must implement this in a subclass")
-
     def remote_complete(self, failure=None):
         """
         Called by the slave's L{buildbot.slave.bot.SlaveBuilder} to
@@ -171,52 +195,6 @@ class RemoteCommand(pb.Referenceable):
             reactor.callLater(0, self._finished, failure)
         return None
 
-    def remoteComplete(self, maybeFailure):
-        return maybeFailure
-
-class LoggedRemoteCommand(RemoteCommand):
-
-    rc = None
-    debug = False
-
-    def __init__(self, remote_command, args, collectStdout=False, **kwargs):
-        self.logs = {}
-        self.delayedLogs = {}
-        self._closeWhenFinished = {}
-        self.collectStdout = collectStdout
-        self.stdout = ''
-        RemoteCommand.__init__(self, remote_command, args, **kwargs)
-
-        self._startTime = None
-        self._remoteElapsed = None
-
-    def __repr__(self):
-        return "<RemoteCommand '%s' at %d>" % (self.remote_command, id(self))
-
-    def useLog(self, loog, closeWhenFinished=False, logfileName=None):
-        assert interfaces.ILogFile.providedBy(loog)
-        if not logfileName:
-            logfileName = loog.getName()
-        assert logfileName not in self.logs
-        assert logfileName not in self.delayedLogs
-        self.logs[logfileName] = loog
-        self._closeWhenFinished[logfileName] = closeWhenFinished
-
-    def useLogDelayed(self, logfileName, activateCallBack, closeWhenFinished=False):
-        assert logfileName not in self.logs
-        assert logfileName not in self.delayedLogs
-        self.delayedLogs[logfileName] = (activateCallBack, closeWhenFinished)
-
-    def _start(self):
-        log.msg("LoggedRemoteCommand._start")
-        if 'stdio' not in self.logs:
-            log.msg("LoggedRemoteCommand (%s) is running a command, but "
-                    "it isn't being logged to anything. This seems unusual."
-                    % self)
-        self.updates = {}
-        self._startTime = util.now()
-        return RemoteCommand._start(self)
-
     def addStdout(self, data):
         if 'stdio' in self.logs:
             self.logs['stdio'].addStdout(data)
@@ -245,7 +223,7 @@ class LoggedRemoteCommand(RemoteCommand):
         else:
             log.msg("%s.addToLog: no such log %s" % (self, logname))
 
-    @metrics.countMethod('LoggedRemoteCommand.remoteUpdate()')
+    @metrics.countMethod('RemoteCommand.remoteUpdate()')
     def remoteUpdate(self, update):
         if self.debug:
             for k,v in update.items():
@@ -280,7 +258,7 @@ class LoggedRemoteCommand(RemoteCommand):
     def remoteComplete(self, maybeFailure):
         if self._startTime and self._remoteElapsed:
             delta = (util.now() - self._startTime) - self._remoteElapsed
-            metrics.MetricTimeEvent.log("LoggedRemoteCommand.overhead", delta)
+            metrics.MetricTimeEvent.log("RemoteCommand.overhead", delta)
 
         for name,loog in self.logs.items():
             if self._closeWhenFinished[name]:
@@ -290,6 +268,7 @@ class LoggedRemoteCommand(RemoteCommand):
                     log.msg("closing log %s" % loog)
                 loog.finish()
         return maybeFailure
+LoggedRemoteCommand = RemoteCommand
 
 
 class LogObserver:
@@ -360,12 +339,12 @@ class LogLineObserver(LogObserver):
         pass
 
 
-class RemoteShellCommand(LoggedRemoteCommand):
+class RemoteShellCommand(RemoteCommand):
     def __init__(self, workdir, command, env=None,
                  want_stdout=1, want_stderr=1,
                  timeout=20*60, maxTime=None, logfiles={},
                  usePTY="slave-config", logEnviron=True,
-                 collectStdout=False):
+                 collectStdout=False, interruptSignal=None):
 
         self.command = command # stash .command, set it later
         if env is not None:
@@ -383,7 +362,9 @@ class RemoteShellCommand(LoggedRemoteCommand):
                 'usePTY': usePTY,
                 'logEnviron': logEnviron,
                 }
-        LoggedRemoteCommand.__init__(self, "shell", args, collectStdout=collectStdout)
+        if interruptSignal is not None:
+            args['interruptSignal'] = interruptSignal
+        RemoteCommand.__init__(self, "shell", args, collectStdout=collectStdout)
 
     def _start(self):
         self.args['command'] = self.command
@@ -395,7 +376,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
         what = "command '%s' in dir '%s'" % (self.args['command'],
                                              self.args['workdir'])
         log.msg(what)
-        return LoggedRemoteCommand._start(self)
+        return RemoteCommand._start(self)
 
     def __repr__(self):
         return "<RemoteShellCommand '%s'>" % repr(self.command)
@@ -409,6 +390,7 @@ class BuildStep(properties.PropertiesMixin):
     warnOnFailure = False
     alwaysRun = False
     doStepIf = True
+    hideStepIf = False
 
     # properties set on a build step are, by nature, always runtime properties
     set_runtime_properties = True
@@ -430,6 +412,7 @@ class BuildStep(properties.PropertiesMixin):
              'progressMetrics',
              'useProgress',
              'doStepIf',
+             'hideStepIf',
              ]
 
     name = "generic"
@@ -619,12 +602,16 @@ class BuildStep(properties.PropertiesMixin):
         if self.progress:
             self.progress.finish()
         self.step_status.stepFinished(results)
+        
+        hidden = self._maybeEvaluate(self.hideStepIf, results, self)
+        self.step_status.setHidden(hidden)
+        
         self.releaseLocks()
         self.deferred.callback(results)
 
     def failed(self, why):
         # This can either be a BuildStepFailed exception/failure, meaning we
-        # should call self.finish, or it can be a real exception, which should
+        # should call self.finished, or it can be a real exception, which should
         # be recorded as such.
         if why.check(BuildStepFailed):
             self.finished(FAILURE)
@@ -640,6 +627,9 @@ class BuildStep(properties.PropertiesMixin):
             self.step_status.setText([self.name, "exception"])
             self.step_status.setText2([self.name])
             self.step_status.stepFinished(EXCEPTION)
+
+            hidden = self._maybeEvaluate(self.hideStepIf, EXCEPTION, self)
+            self.step_status.setHidden(hidden)
         except:
             log.msg("exception during failure processing")
             log.err()
@@ -722,7 +712,12 @@ class BuildStep(properties.PropertiesMixin):
         c.buildslave = self.buildslave
         d = c.run(self, self.remote)
         return d
-
+    
+    @staticmethod
+    def _maybeEvaluate(value, *args, **kwargs):
+        if callable(value):
+            value = value(*args, **kwargs)
+        return value
 
 components.registerAdapter(
         lambda step : interfaces.IProperties(step.build),
@@ -756,16 +751,18 @@ class LoggingBuildStep(BuildStep):
                                  lazylogfiles=lazylogfiles,
                                  log_eval_func=log_eval_func)
 
-        if logfiles:
-            assert type(logfiles) is type({}), \
-                "the ShellCommand 'logfiles' parameter must be a dictionary"
+        if logfiles and not isinstance(logfiles, dict):
+            config.error(
+                "the ShellCommand 'logfiles' parameter must be a dictionary")
 
         # merge a class-level 'logfiles' attribute with one passed in as an
         # argument
         self.logfiles = self.logfiles.copy()
         self.logfiles.update(logfiles)
         self.lazylogfiles = lazylogfiles
-        assert not log_eval_func or callable(log_eval_func)
+        if log_eval_func and not callable(log_eval_func):
+            config.error(
+                "the 'log_eval_func' paramater must be a callable")
         self.log_eval_func = log_eval_func
         self.addLogObserver('stdio', OutputProgressObserver("output"))
 
@@ -813,7 +810,7 @@ class LoggingBuildStep(BuildStep):
     def setupLogfiles(self, cmd, logfiles):
         for logname,remotefilename in logfiles.items():
             if self.lazylogfiles:
-                # Ask LoggedRemoteCommand to watch a logfile, but only add
+                # Ask RemoteCommand to watch a logfile, but only add
                 # it when/if we see any data.
                 #
                 # The dummy default argument local_logname is a work-around for
@@ -824,7 +821,7 @@ class LoggingBuildStep(BuildStep):
             else:
                 # tell the BuildStepStatus to add a LogFile
                 newlog = self.addLog(logname)
-                # and tell the LoggedRemoteCommand to feed it
+                # and tell the RemoteCommand to feed it
                 cmd.useLog(newlog, True)
 
     def interrupt(self, reason):
