@@ -1,6 +1,20 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 import sys
 import re
-import exceptions
 from twisted.python import log
 from twisted.internet import defer
 from twisted.enterprise import adbapi
@@ -252,6 +266,8 @@ class MTR(Test):
         dbpool is specified. The test_type string, if specified, will also
         appear on the waterfall page."""
 
+    renderables = [ 'mtr_subdir' ]
+
     def __init__(self, dbpool=None, test_type=None, test_info="",
                  description=None, descriptionDone=None,
                  autoCreateTables=False, textLimit=5, testNameLimit=16,
@@ -290,9 +306,6 @@ class MTR(Test):
                                  mtr_subdir=self.mtr_subdir)
 
     def start(self):
-        properties = self.build.getProperties()
-        subdir = properties.render(self.mtr_subdir)
-
         # Add mysql server logfiles.
         for mtr in range(0, self.parallel+1):
             for mysqld in range(1, 4+1):
@@ -302,7 +315,7 @@ class MTR(Test):
                 else:
                     logname = "mysqld.%d.err.%d" % (mysqld, mtr)
                     filename = "var/%d/log/mysqld.%d.err" % (mtr, mysqld)
-                self.addLogFile(logname, subdir + "/" + filename)
+                self.addLogFile(logname, self.mtr_subdir + "/" + filename)
 
         self.myMtr = self.MyMtrLogObserver(textLimit=self.textLimit,
                                            testNameLimit=self.testNameLimit,
@@ -409,10 +422,8 @@ CREATE TABLE IF NOT EXISTS test_warnings(
 ) ENGINE=innodb
 """)
 
-        revision = None
-        try:
-            revision = self.getProperty("got_revision")
-        except exceptions.KeyError:
+        revision = self.getProperty("got_revision")
+        if revision is None:
             revision = self.getProperty("revision")
         typ = "mtr"
         if self.test_type:
