@@ -13,16 +13,14 @@
 #
 # Copyright Buildbot Team Members
 
+from zope.interface import implements
 from twisted.python import failure, log
 from twisted.application import service
 from twisted.internet import defer
 from buildbot.process.properties import Properties
 from buildbot.util import ComparableMixin
 from buildbot.changes import changes
-
-def isScheduler(sch):
-    "Check that an object is a scheduler; used for configuration checks."
-    return isinstance(sch, BaseScheduler)
+from buildbot import config, interfaces
 
 class BaseScheduler(service.MultiService, ComparableMixin):
     """
@@ -34,6 +32,8 @@ class BaseScheduler(service.MultiService, ComparableMixin):
     C{base.Scheduler.compare_attrs}.
     """
 
+    implements(interfaces.IScheduler)
+
     compare_attrs = ('name', 'builderNames', 'properties')
 
     def __init__(self, name, builderNames, properties):
@@ -60,11 +60,18 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         self.name = name
         "name of this scheduler; used to identify replacements on reconfig"
 
-        errmsg = ("The builderNames argument to a scheduler must be a list "
+        ok = True
+        if not isinstance(builderNames, (list, tuple)):
+            ok = False
+        else:
+            for b in builderNames:
+                if not isinstance(b, basestring):
+                    ok = False
+        if not ok:
+            config.error(
+                "The builderNames argument to a scheduler must be a list "
                   "of Builder names.")
-        assert isinstance(builderNames, (list, tuple)), errmsg
-        for b in builderNames:
-            assert isinstance(b, basestring), errmsg
+
         self.builderNames = builderNames
         "list of builder names to start in each buildset"
 
@@ -73,13 +80,9 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         self.properties.update(properties, "Scheduler")
         self.properties.setProperty("scheduler", name, "Scheduler")
 
-        self.schedulerid = None
-        """ID of this scheduler; set just before the scheduler starts, and set
-        to None after stopService is complete."""
+        self.objectid = None
 
         self.master = None
-        """BuildMaster instance; set just before the scheduler starts, and set
-        to None after stopService is complete."""
 
         # internal variables
         self._change_subscription = None
@@ -88,24 +91,17 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
     ## service handling
 
-    def _setUpScheduler(self, schedulerid, master, manager):
-        # this is called by SchedulerManager *before* startService
-        self.schedulerid = schedulerid
-        self.master = master
-
     def startService(self):
         service.MultiService.startService(self)
 
+    def findNewSchedulerInstance(self, new_config):
+        return new_config.schedulers[self.name] # should exist!
+
     def stopService(self):
         d = defer.maybeDeferred(self._stopConsumingChanges)
         d.addCallback(lambda _ : service.MultiService.stopService(self))
         return d
 
-    def _shutDownScheduler(self):
-        # called by SchedulerManager *after* stopService is complete
-        self.schedulerid = None
-        self.master = None
-
     ## state management
 
     @defer.deferredGenerator
@@ -258,6 +254,7 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
     ## starting bulids
 
+    @defer.deferredGenerator
     def addBuildsetForLatest(self, reason='', external_idstring=None,
                         branch=None, repository='', project='',
                         builderNames=None, properties=None):
@@ -283,15 +280,26 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         @type properties: L{buildbot.process.properties.Properties}
         @returns: (buildset ID, buildrequest IDs) via Deferred
         """
-        d = self.master.db.sourcestamps.addSourceStamp(
+        # Define setid for this set of changed repositories
+        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet())
+        yield wfd
+        setid = wfd.getResult()
+
+        wfd = defer.waitForDeferred(self.master.db.sourcestamps.addSourceStamp(
                 branch=branch, revision=None, repository=repository,
-                project=project)
-        d.addCallback(self.addBuildsetForSourceStamp, reason=reason,
+                project=project, sourcestampsetid=setid))
+        yield wfd
+        wfd.getResult()
+
+        wfd = defer.waitForDeferred(self.addBuildsetForSourceStamp(
+                                setid=setid, reason=reason,
                                 external_idstring=external_idstring,
                                 builderNames=builderNames,
-                                properties=properties)
-        return d
+                                properties=properties))
+        yield wfd
+        yield wfd.getResult()
 
+    @defer.deferredGenerator
     def addBuildsetForChanges(self, reason='', external_idstring=None,
             changeids=[], builderNames=None, properties=None):
         """
@@ -318,27 +326,127 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
         # attributes for this sourcestamp will be based on the most recent
         # change, so fetch the change with the highest id
-        d = self.master.db.changes.getChange(max(changeids))
-        def chdict2change(chdict):
-            if not chdict:
-                return None
-            return changes.Change.fromChdict(self.master, chdict)
-        d.addCallback(chdict2change)
-        def create_sourcestamp(change):
-            return self.master.db.sourcestamps.addSourceStamp(
+        wfd = defer.waitForDeferred(self.master.db.changes.getChange(max(changeids)))
+        yield wfd
+        chdict = wfd.getResult()
+
+        change = None
+        if chdict:
+            wfd = defer.waitForDeferred(changes.Change.fromChdict(self.master, chdict))
+            yield wfd
+            change = wfd.getResult()
+
+        # Define setid for this set of changed repositories
+        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet())
+        yield wfd
+        setid = wfd.getResult()
+
+        wfd = defer.waitForDeferred(self.master.db.sourcestamps.addSourceStamp(
                     branch=change.branch,
                     revision=change.revision,
                     repository=change.repository,
                     project=change.project,
-                    changeids=changeids)
-        d.addCallback(create_sourcestamp)
-        d.addCallback(self.addBuildsetForSourceStamp, reason=reason,
+                    changeids=changeids,
+                    sourcestampsetid=setid))
+        yield wfd
+        wfd.getResult()
+
+        wfd = defer.waitForDeferred(self.addBuildsetForSourceStamp(
+                                setid=setid, reason=reason,
                                 external_idstring=external_idstring,
                                 builderNames=builderNames,
-                                properties=properties)
-        return d
+                                properties=properties))
+        yield wfd
 
-    def addBuildsetForSourceStamp(self, ssid, reason='', external_idstring=None,
+        yield wfd.getResult()
+
+    @defer.deferredGenerator
+    def addBuildsetForChangesMultiRepo(self, reason='', external_idstring=None,
+            changeids=[], builderNames=None, properties=None):
+        assert changeids is not []
+        chDicts = {}
+
+        def getChange(changeid = None):
+            d = self.master.db.changes.getChange(changeid)
+            def chdict2change(chdict):
+                if not chdict:
+                    return None
+                return changes.Change.fromChdict(self.master, chdict)
+            d.addCallback(chdict2change)
+
+        def groupChange(change):
+            if change.repository not in chDicts:
+                chDicts[change.repository] = []
+            chDicts[change.repository].append(change)
+
+        def get_changeids_from_repo(repository):
+            changeids = []
+            for change in chDicts[repository]:
+                changeids.append(change.number)
+            return changeids
+
+        def create_sourcestamp(changeids, change = None, setid = None):
+            def add_sourcestamp(setid, changeids = None):
+                return self.master.db.sourcestamps.addSourceStamp(
+                        branch=change.branch,
+                        revision=change.revision,
+                        repository=change.repository,
+                        project=change.project,
+                        changeids=changeids,
+                        setid=setid)
+            d.addCallback(add_sourcestamp, setid = setid, changeids = changeids)
+            return d
+
+        def create_sourcestamp_without_changes(setid, repository):
+            return self.master.db.sourcestamps.addSourceStamp(
+                    branch=self.default_branch,
+                    revision=None,
+                    repository=repository,
+                    project=self.default_project,
+                    changeids=changeids,
+                    sourcestampsetid=setid)
+
+        d = defer.Deferred()
+        if self.repositories is None:
+            # attributes for this sourcestamp will be based on the most recent
+            # change, so fetch the change with the highest id (= old single
+            # sourcestamp functionality)
+            d.addCallBack(getChange,changeid=max(changeids))
+            d.addCallBack(groupChange)
+        else:
+            for changeid in changeids:
+                d.addCallBack(getChange,changeid = changeid)
+                d.addCallBack(groupChange)
+
+        # Define setid for this set of changed repositories
+        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet)
+        yield wfd
+        setid = wfd.getResult()
+
+        #process all unchanged repositories
+        if self.repositories is not None:
+            for repo in self.repositories:
+                if repo not in chDicts:
+                    # repository was not changed
+                    # call create_sourcestamp
+                    d.addCallback(create_sourcestamp_without_changes, setid, repo)
+
+        # process all changed
+        for repo in chDicts:
+            d.addCallback(get_changeids_from_repo, repository = repo)
+            d.addCallback(create_sourcestamp, setid = setid, change=chDicts[repo][-1])
+
+        # add one buildset, this buildset is connected to the sourcestamps by the setid
+        d.addCallback(self.addBuildsetForSourceStamp, setid=setid, reason=reason,
+                            external_idstring=external_idstring,
+                            builderNames=builderNames,
+                            properties=properties)
+
+        yield d
+
+
+    @defer.deferredGenerator
+    def addBuildsetForSourceStamp(self, ssid=None, setid=None, reason='', external_idstring=None,
             properties=None, builderNames=None):
         """
         Add a buildset for the given, already-existing sourcestamp.
@@ -356,8 +464,12 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         @type properties: L{buildbot.process.properties.Properties}
         @param builderNames: builders to name in the buildset (defaults to
             C{self.builderNames})
+        @param setid: idenitification of a set of sourcestamps
         @returns: (buildset ID, buildrequest IDs) via Deferred
         """
+        assert (ssid is None and setid is not None) \
+            or (ssid is not None and setid is None), "pass a single sourcestamp OR set not both"
+
         # combine properties
         if properties:
             properties.updateFromProperties(self.properties)
@@ -372,7 +484,20 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         # addBuildset method
         properties_dict = properties.asDict()
 
-        # add the buildset
-        return self.master.addBuildset(
-                ssid=ssid, reason=reason, properties=properties_dict,
-                builderNames=builderNames, external_idstring=external_idstring)
+        if setid == None:
+            if ssid != None:
+                wfd = defer.waitForDeferred(self.master.db.sourcestamps.getSourceStamp(ssid))
+                yield wfd
+                ssdict = wfd.getResult()
+                setid = ssdict['sourcestampsetid']
+            else:
+                # no sourcestamp and no sets
+                yield None
+
+        wfd = defer.waitForDeferred(self.master.addBuildset(
+                                        sourcestampsetid=setid, reason=reason,
+                                        properties=properties_dict,
+                                        builderNames=builderNames,
+                                        external_idstring=external_idstring))
+        yield wfd
+        yield wfd.getResult()
