@@ -1,4 +1,18 @@
-# -*- test-case-name: buildbot.test.test_status -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 from twisted.spread import pb
 from twisted.python import components, log as twlog
@@ -8,7 +22,7 @@ from twisted.cred import portal, checkers
 
 from buildbot import interfaces
 from zope.interface import Interface, implements
-from buildbot.status import builder, base
+from buildbot.status import logfile, base
 from buildbot.changes import changes
 
 class IRemote(Interface):
@@ -36,21 +50,21 @@ class RemoteBuildSet(pb.Referenceable):
         return self.b.getID()
 
     def remote_getBuilderNames(self):
-        return self.b.getBuilderNames()
+        return self.b.getBuilderNames() # note: passes along the Deferred
 
     def remote_getBuildRequests(self):
         """Returns a list of (builderName, BuildRequest) tuples."""
-        return [(br.getBuilderName(), IRemote(br))
-                for br in self.b.getBuildRequests()]
+        d = self.b.getBuilderNamesAndBuildRequests()
+        def add_remote(buildrequests):
+            for k,v in buildrequests.iteritems():
+                buildrequests[k] = IRemote(v)
+            return buildrequests.items()
+        d.addCallback(add_remote)
+        return d
 
     def remote_isFinished(self):
         return self.b.isFinished()
 
-    def remote_waitUntilSuccess(self):
-        d = self.b.waitUntilSuccess()
-        d.addCallback(lambda res: self)
-        return d
-
     def remote_waitUntilFinished(self):
         d = self.b.waitUntilFinished()
         d.addCallback(lambda res: self)
@@ -101,9 +115,12 @@ components.registerAdapter(RemoteBuilder,
 class RemoteBuildRequest(pb.Referenceable):
     def __init__(self, buildreq):
         self.b = buildreq
+        # mapping of observers (RemoteReference instances) to local callable
+        # objects that have been passed to BuildRequestStatus.subscribe
         self.observers = []
 
     def remote_getSourceStamp(self):
+        # note that this now returns a Deferred
         return self.b.getSourceStamp()
 
     def remote_getBuilderName(self):
@@ -113,22 +130,20 @@ class RemoteBuildRequest(pb.Referenceable):
         """The observer's remote_newbuild method will be called (with two
         arguments: the RemoteBuild object, and our builderName) for each new
         Build that is created to handle this BuildRequest."""
-        self.observers.append(observer)
         def send(bs):
             d = observer.callRemote("newbuild",
                                     IRemote(bs), self.b.getBuilderName())
-            d.addErrback(lambda err: None)
-        reactor.callLater(0, self.b.subscribe, send)
+            d.addErrback(twlog.err,
+                    "while calling client-side remote_newbuild")
+        self.observers.append((observer, send))
+        self.b.subscribe(send)
 
     def remote_unsubscribe(self, observer):
-        # PB (well, at least oldpb) doesn't re-use RemoteReference instances,
-        # so sending the same object across the wire twice will result in two
-        # separate objects that compare as equal ('a is not b' and 'a == b').
-        # That means we can't use a simple 'self.observers.remove(observer)'
-        # here.
-        for o in self.observers:
-            if o == observer:
-                self.observers.remove(o)
+        for i, (obs, send) in enumerate(self.observers):
+            if obs == observer:
+                del self.observers[i]
+                self.b.unsubscribe(send)
+                break
 
 components.registerAdapter(RemoteBuildRequest,
                            interfaces.IBuildRequestStatus, IRemote)    
@@ -150,6 +165,9 @@ class RemoteBuild(pb.Referenceable):
     def remote_getChanges(self):
         return [IRemote(c) for c in self.b.getChanges()]
 
+    def remote_getRevisions(self):
+        return self.b.getRevisions()
+
     def remote_getResponsibleUsers(self):
         return self.b.getResponsibleUsers()
 
@@ -323,7 +341,7 @@ class RemoteLog(pb.Referenceable):
         return self.l.getChunks()
     # TODO: subscription interface
 
-components.registerAdapter(RemoteLog, builder.LogFile, IRemote)
+components.registerAdapter(RemoteLog, logfile.LogFile, IRemote)
 # TODO: something similar for builder.HTMLLogfile ?
 
 class RemoteChange:
@@ -412,7 +430,11 @@ class StatusClientPerspective(base.StatusReceiverPerspective):
     def perspective_getBuildSets(self):
         """This returns tuples of (buildset, bsid), because that is much more
         convenient for tryclient."""
-        return [(IRemote(s), s.getID()) for s in self.status.getBuildSets()]
+        d = self.status.getBuildSets()
+        def make_remotes(buildsets):
+            return [(IRemote(s), s.id) for s in buildsets]
+        d.addCallback(make_remotes)
+        return d
 
     def perspective_getBuilderNames(self):
         return self.status.getBuilderNames()
@@ -554,10 +576,7 @@ class PBListener(base.StatusReceiverMultiService):
 
     def setServiceParent(self, parent):
         base.StatusReceiverMultiService.setServiceParent(self, parent)
-        self.setup()
-
-    def setup(self):
-        self.status = self.parent.getStatus()
+        self.status = parent
 
     def requestAvatar(self, avatarID, mind, interface):
         assert interface == pb.IPerspective
