@@ -16,7 +16,9 @@
 
 import os
 from zope.interface import Interface, Attribute, implements
-from buildbot.status.web.base import HtmlResource
+from buildbot.status.web.base import HtmlResource, ActionResource
+from buildbot.status.web.base import path_to_authfail
+
 from buildbot.process.users import users
 
 class IAuth(Interface):
@@ -32,6 +34,11 @@ class IAuth(Interface):
     def authenticate(self, user, passwd):
             """Check whether C{user} / C{passwd} are valid."""
 
+    def getUserInfo(self, user):
+            """return dict with user info.
+            dict( fullName="", email="", groups=[])
+            """
+
     def errmsg(self):
             """Get the reason authentication failed."""
 
@@ -42,6 +49,10 @@ class AuthBase:
     def errmsg(self):
         return self.err
 
+    def getUserInfo(self, user):
+        """default dummy impl"""
+        return dict(userName=user, fullName=user, email=user+"@localhost", groups=[ user ])
+
 class BasicAuth(AuthBase):
     implements(IAuth)
     """Implement basic authentication against a list of user/passwd."""
@@ -52,7 +63,7 @@ class BasicAuth(AuthBase):
     def __init__(self, userpass):
         """C{userpass} is a list of (user, passwd)."""
         for item in userpass:
-            assert isinstance(item, tuple)
+            assert isinstance(item, tuple) or isinstance(item, list)
             u, p = item
             assert isinstance(u, str)
             assert isinstance(p, str)
@@ -138,6 +149,39 @@ class AuthFailResource(HtmlResource):
     pageTitle = "Authentication Failed"
 
     def content(self, request, cxt):
-        template = request.site.buildbot_service.templates.get_template("authfail.html")
+        templates =request.site.buildbot_service.templates
+        template = templates.get_template("authfail.html") 
         return template.render(**cxt)
-    
+
+class AuthzFailResource(HtmlResource):
+    pageTitle = "Authorization Failed"
+
+    def content(self, request, cxt):
+        templates =request.site.buildbot_service.templates
+        template = templates.get_template("authzfail.html") 
+        return template.render(**cxt)
+
+class LoginResource(ActionResource):
+
+    def performAction(self, request):
+        authz = self.getAuthz(request)
+        d = authz.login(request)
+        def on_login(res):
+            if res:
+                status = request.site.buildbot_service.master.status
+                root = status.getBuildbotURL()
+                return request.requestHeaders.getRawHeaders('referer',
+                                                            [root])[0]
+            else:
+                return path_to_authfail(request)
+        d.addBoth(on_login)
+        return d
+
+class LogoutResource(ActionResource):
+
+    def performAction(self, request):
+        authz = self.getAuthz(request)
+        authz.logout(request)
+        status = request.site.buildbot_service.master.status
+        root = status.getBuildbotURL()
+        return request.requestHeaders.getRawHeaders('referer',[root])[0]
