@@ -44,8 +44,6 @@ class BuildStepMixin(object):
         remotecommand.FakeRemoteCommand.testcase = self
         self.patch(buildstep, 'RemoteCommand',
                 remotecommand.FakeRemoteCommand)
-        self.patch(buildstep, 'LoggedRemoteCommand',
-                remotecommand.FakeLoggedRemoteCommand)
         self.patch(buildstep, 'RemoteShellCommand',
                 remotecommand.FakeRemoteShellCommand)
         self.expected_remote_commands = []
@@ -160,6 +158,7 @@ class BuildStepMixin(object):
         self.exp_properties = {}
         self.exp_missing_properties = []
         self.exp_logfiles = {}
+        self.exp_hidden = False
 
         return step
 
@@ -194,6 +193,12 @@ class BuildStepMixin(object):
         Expect a logfile with the given contents
         """
         self.exp_logfiles[logfile] = contents
+    
+    def expectHidden(self, hidden):
+        """
+        Set whether the step is expected to be hidden.
+        """
+        self.exp_hidden = hidden
 
     def runStep(self):
         """
@@ -220,6 +225,7 @@ class BuildStepMixin(object):
                 self.failIf(self.properties.hasProperty(pn))
             for log, contents in self.exp_logfiles.iteritems():
                 self.assertEqual(self.step_status.logs[log].stdout, contents)
+            self.step_status.setHidden.assert_called_once_with(self.exp_hidden)
         d.addCallback(check)
         return d
 
