@@ -31,21 +31,40 @@ import sys
 
 from twisted.spread import pb
 from twisted.cred import credentials
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 
-from buildbot.scripts import runner
 from optparse import OptionParser
 
-# Modify this to fit your setup, or pass in --master server:host on the
+# Modify this to fit your setup, or pass in --master server:port on the
 # command line
 
 master = "localhost:9989"
 
-# When sending the notification, send this category iff
+# When sending the notification, send this category if (and only if)
 # it's set (via --category)
 
 category = None
 
+# When sending the notification, send this repository if (and only if)
+# it's set (via --repository)
+
+repository = None
+
+# When sending the notification, send this project if (and only if)
+# it's set (via --project)
+
+project = None
+
+# Username portion of PB login credentials to send the changes to the master
+username = "change"
+
+# Password portion of PB login credentials to send the changes to the master
+auth = "changepw"
+
+# When converting strings to unicode, assume this encoding. 
+# (set with --encoding)
+
+encoding = 'utf8'
 
 # The GIT_DIR environment variable must have been set up so that any
 # git commands that are executed will operate on the repository we're
@@ -53,37 +72,45 @@ category = None
 
 changes = []
 
-
 def connectFailed(error):
     logging.error("Could not connect to %s: %s"
             % (master, error.getErrorMessage()))
     return error
 
 
-def addChange(remote, changei):
-    logging.debug("addChange %s, %s" % (repr(remote), repr(changei)))
-    try:
-        c = changei.next()
-    except StopIteration:
-        remote.broker.transport.loseConnection()
-        return None
+def addChanges(remote, changei, src='git'):
+    logging.debug("addChanges %s, %s" % (repr(remote), repr(changei)))
+    def addChange(c):
+        logging.info("New revision: %s" % c['revision'][:8])
+        for key, value in c.iteritems():
+            logging.debug("  %s: %s" % (key, value))
 
-    logging.info("New revision: %s" % c['revision'][:8])
-    for key, value in c.iteritems():
-        logging.debug("  %s: %s" % (key, value))
+        c['src'] = src
+        d = remote.callRemote('addChange', c)
+        return d
 
-    d = remote.callRemote('addChange', c)
+    finished_d = defer.Deferred()
+    def iter():
+        try:
+            c = changei.next()
+            d = addChange(c)
+            # handle successful completion by re-iterating, but not immediately
+            # as that will blow out the Python stack
+            def cb(_):
+                reactor.callLater(0, iter)
+            d.addCallback(cb)
+            # and pass errors along to the outer deferred
+            d.addErrback(finished_d.errback)
+        except StopIteration:
+            remote.broker.transport.loseConnection()
+            finished_d.callback(None)
 
-    # tail recursion in Twisted can blow out the stack, so we
-    # insert a callLater to delay things
-    def recurseLater(x):
-        reactor.callLater(0, addChange, remote, changei)
-    d.addCallback(recurseLater)
-    return d
+    iter()
+    return finished_d
 
 
 def connected(remote):
-    return addChange(remote, changes.__iter__())
+    return addChanges(remote, changes.__iter__())
 
 
 def grab_commit_info(c, rev):
@@ -91,26 +118,31 @@ def grab_commit_info(c, rev):
     f = os.popen("git show --raw --pretty=full %s" % rev, 'r')
 
     files = []
+    comments = []
 
     while True:
         line = f.readline()
         if not line:
             break
 
+        if line.startswith(4*' '):
+            comments.append(line[4:])
+
         m = re.match(r"^:.*[MAD]\s+(.+)$", line)
         if m:
             logging.debug("Got file: %s" % m.group(1))
-            files.append(m.group(1))
+            files.append(unicode(m.group(1), encoding=encoding))
             continue
 
         m = re.match(r"^Author:\s+(.+)$", line)
         if m:
             logging.debug("Got author: %s" % m.group(1))
-            c['who'] = m.group(1)
+            c['who'] = unicode(m.group(1), encoding=encoding)
 
         if re.match(r"^Merge: .*$", line):
             files.append('merge')
 
+    c['comments'] = ''.join(comments)
     c['files'] = files
     status = f.close()
     if status:
@@ -127,11 +159,18 @@ def gen_changes(input, branch):
 
         m = re.match(r"^([0-9a-f]+) (.*)$", line.strip())
         c = {'revision': m.group(1),
-             'comments': m.group(2),
-             'branch': branch,
+             'branch': unicode(branch, encoding=encoding),
         }
+
         if category:
-            c['category'] = category
+            c['category'] = unicode(category, encoding=encoding)
+
+        if repository:
+            c['repository'] = unicode(repository, encoding=encoding)
+
+        if project:
+            c['project'] = unicode(project, encoding=encoding)
+
         grab_commit_info(c, m.group(1))
         changes.append(c)
 
@@ -175,7 +214,7 @@ def gen_update_branch_changes(oldrev, newrev, refname, branch):
     if baserev != oldrev:
         c = {'revision': baserev,
              'comments': "Rewind branch",
-             'branch': branch,
+             'branch': unicode(branch, encoding=encoding),
              'who': "dummy",
         }
         logging.info("Branch %s was rewound to %s" % (branch, baserev[:8]))
@@ -186,16 +225,22 @@ def gen_update_branch_changes(oldrev, newrev, refname, branch):
             if not line:
                 break
 
-            file = re.match(r"^:.*[MAD]\s*(.+)$", line).group(1)
+            file = re.match(r"^:.*[MAD]\s+(.+)$", line).group(1)
             logging.debug("  Rewound file: %s" % file)
-            files.append(file)
+            files.append(unicode(file, encoding=encoding))
 
         status = f.close()
         if status:
             logging.warning("git diff exited with status %d" % status)
 
         if category:
-            c['category'] = category
+            c['category'] = unicode(category, encoding=encoding)
+
+        if repository:
+            c['repository'] = unicode(repository, encoding=encoding)
+
+        if project:
+            c['project'] = unicode(project, encoding=encoding)
 
         if files:
             c['files'] = files
@@ -252,7 +297,7 @@ def process_changes():
     port = int(port)
 
     f = pb.PBClientFactory()
-    d = f.login(credentials.UsernamePassword("change", "changepw"))
+    d = f.login(credentials.UsernamePassword(username, auth))
     reactor.connectTCP(host, port, f)
 
     d.addErrback(connectFailed)
@@ -274,6 +319,24 @@ def parse_options():
             help=master_help)
     parser.add_option("-c", "--category", action="store",
                       type="string", help="Scheduler category to notify.")
+    parser.add_option("-r", "--repository", action="store",
+                      type="string", help="Git repository URL to send.")
+    parser.add_option("-p", "--project", action="store",
+                      type="string", help="Project to send.")
+    encoding_help = ("Encoding to use when converting strings to "
+                     "unicode. Default is %(encoding)s." % 
+                     { "encoding" : encoding })
+    parser.add_option("-e", "--encoding", action="store", type="string", 
+                      help=encoding_help)
+    username_help = ("Username used in PB connection auth, defaults to "
+                     "%(username)s." % { "username" : username })
+    parser.add_option("-u", "--username", action="store", type="string",
+                      help=username_help)
+    auth_help = ("Password used in PB connection auth, defaults to "
+                     "%(auth)s." % { "auth" : auth })
+    # 'a' instead of 'p' due to collisions with the project short option
+    parser.add_option("-a", "--auth", action="store", type="string",
+                      help=auth_help)
     options, args = parser.parse_args()
     return options
 
@@ -308,6 +371,21 @@ try:
     if options.category:
         category = options.category
 
+    if options.repository:
+        repository = options.repository
+
+    if options.project:
+        project = options.project
+
+    if options.username:
+        username = options.username
+
+    if options.auth:
+        auth = options.auth
+
+    if options.encoding:
+        encoding = options.encoding
+
     process_changes()
 except SystemExit:
     pass
